<?php
/**
 * REST API: Wooglop_REST_Glop_Store_Controller class
 *
 * @package Glop
 * @subpackage wooglop
 * @since 1.0.0
 */

/**
 * Core class to GLOP via the REST API.
 *
 * @since 1.0.0
 *
 * @see Wooglop_WP_REST_Controller
 */
class Wooglop_REST_Glop_Store_Controller extends Wooglop_WP_REST_Controller {

	/**
	 * Registers the routes for the objects of the controller.
	 *
	 * @since 1.0.0
	 *
	 * @see register_rest_route()
	 */
	public function register_routes() {
		$this->rest_base = 'store';
		register_rest_route(
			$this->namespace,
			'/' . $this->rest_base,
			array(
				'args'   => array(),
				array(
					'methods'             => WP_REST_Server::EDITABLE,
					'callback'            => array( $this, 'update_store_options' ),
					'permission_callback' => array( $this, 'get_items_permissions_check_no_key' ),
					'args'                => $this->get_endpoint_args_for_item_schema( WP_REST_Server::EDITABLE ),
				),
				'schema' => array( $this, 'get_public_item_schema' ),
			)
		);
	}

	/**
	 * Crear JSON object
	 *
	 * @since 1.0.0
	 *
	 * @param  WP_REST_Request $request Full details about the request.
	 * @return WP_REST_Response|WP_Error Response object on success, or WP_Error object on failure.
	 */
	public function update_store_options( $request ) {
		try {
			$this->log_request( $request );
			$options = $request->get_param( 'options' );

			// DANIEL RUIZ: 03 Agosto 2021 #2331
			$options = array_change_key_case( $options, CASE_LOWER );

			if ( isset( $options['aviso'] ) ) {
				update_option( 'woocommerce_demo_store',
					0 === intval( $options['aviso'] ) ? 'no' : 'yes' );
			}

			if ( isset( $options['aviso_mensaje'] ) ) {
				update_option( 'woocommerce_demo_store_notice', esc_html( $options['aviso_mensaje'] ) );
			}

			if ( isset( $options['cerrar_tienda'] ) ) {
				update_option( 'byconsolewooodt_cerrar_tienda',
					1 === intval( $options['cerrar_tienda'] ) ? 'on' : '' );
			}

			if ( isset( $options['delivery_time'] ) ) {
				update_option( 'byconsolewooodt_chekout_page_section_heading', esc_html( $options['delivery_time'] ) );
			}

			if ( isset( $options['mostrar_imagenes'] ) ) {
				update_option( 'wglop_ocultar_imagen_producto', 1 === intval( $options['mostrar_imagenes'] ) ? 'no' : 'yes' );
			}

			if ( isset( $options['top_ventas_enabled'] ) ) {
				update_option( 'wglop_top_ventas_enabled', 1 === intval( $options['top_ventas_enabled'] ) ? 'yes' : 'no' );
			}

			if ( isset( $options['caracteres_descripcion_corta'] ) ) {
				update_option( 'wglop_n_caracteres_cortos', absint( $options['caracteres_descripcion_corta'] ) );
			}

			if ( isset( $options['texto_boton_add_to_cart'] ) ) {
				update_option( 'wglop_texto_boton_add_to_cart', esc_html( $options['texto_boton_add_to_cart'] ) );
			}

			if ( isset( $options['texto_boton_finalize_order'] ) ) {
				update_option( 'wglop_texto_boton_finalizar_compra', esc_html( $options['texto_boton_finalize_order'] ) );
			}

			if ( isset( $options['texto_seccion_top_ventas'] ) ) {
				update_option( 'wglop_texto_seccion_top_ventas', esc_html( $options['texto_seccion_top_ventas'] ) );
			}

			if ( isset( $options['descripcion_seccion_top_ventas'] ) ) {
				update_option( 'wglop_texto_descripcion_top_ventas', esc_html( $options['descripcion_seccion_top_ventas'] ) );
			}

			if ( isset( $options['orden_categorias'] ) ) {
				$categorias = $options['orden_categorias'];
				$this->reordenar_categorias( $categorias );
			}

			if ( isset( $options['top_ventas_products'] ) ) {
				$productos = $options['top_ventas_products'];
				$this->set_top_ventas( $productos );
			}

			if ( isset( $options['alergenos'] ) ) {
				$alergenos = $options['alergenos'];
				$this->set_alergenos( $alergenos );
			}

			if( isset( $options['cw_ocultar_imagen_producto'] ) ) {
				update_option( 'wglop_ocultar_imagen_producto', 1 === intval( $options['cw_ocultar_imagen_producto'] ) ? 'yes' : 'no' );
			}

			if( isset( $options['cw_activar_top_ventas'] ) ) {
				update_option( 'wglop_top_ventas_enabled', 1 === intval( $options['cw_activar_top_ventas'] ) ? 'yes' : 'no' );
			}

			if( isset( $options['cw_cambiar_icono_idiomas'] ) ) {
				update_option( 'wglop_languages_enabled', 1 === intval( $options['cw_cambiar_icono_idiomas'] ) ? 'yes' : 'no' );
			}

			if( isset( $options['cw_activar_vista_minimalista'] ) ) {
				update_option( 'wglop_minimalista_enabled', 1 === intval( $options['cw_activar_vista_minimalista'] ) ? 'yes' : 'no' );
			}

			if( isset( $options['cw_mostrar_descripcion_corta'] ) ) {
				update_option( 'wglop_minimalista_mostrar_descripcion_corta_enabled', 1 === intval( $options['cw_mostrar_descripcion_corta'] ) ? 'yes' : 'no' );
			}

			if( isset( $options['cw_mostrar_alergenos'] ) ) {
				update_option( 'wglop_minimalista_mostrar_alergenos_enabled', 1 === intval( $options['cw_mostrar_alergenos'] ) ? 'yes' : 'no' );
			}

			if( isset( $options['cw_mostrar_detalle_articulos'] ) ) {
				update_option( 'wglop_minimalista_ocultar_detalles_articulos_enabled', 1 === intval( $options['cw_mostrar_detalle_articulos'] ) ? 'no' : 'yes' );
			}





			if( isset( $options['cw_texto_header_titulo'] ) ) {
				update_option( 'wglop_header_titulo', $options['cw_texto_header_titulo'] );
			}

			if( isset( $options['cw_texto_header_descripcion'] ) ) {
				update_option( 'wglop_header_descripcion', $options['cw_texto_header_descripcion'] );
			}

			if( isset( $options['cw_texto_boton_pedir_comida'] ) ) {
				update_option( 'wglop_header_texto_pide_comida', $options['cw_texto_boton_pedir_comida'] );
			}

			if( isset( $options['cw_texto_boton_finalizar_compra'] ) ) {
				update_option( 'wglop_header_texto_finalizar_compra', $options['cw_texto_boton_finalizar_compra'] );
			}

			if( isset( $options['cw_texto_seccion_top_ventas'] ) ) {
				update_option( 'wglop_texto_seccion_top_ventas', $options['cw_texto_seccion_top_ventas'] );
			}

			if( isset( $options['cw_descr_seccion_top_ventas'] ) ) {
				update_option( 'wglop_texto_descripcion_top_ventas', $options['cw_descr_seccion_top_ventas'] );
			}

			if( isset( $options['cw_caracteres_descripcion_corta'] ) ) {
				update_option( 'wglop_n_caracteres_cortos', $options['cw_caracteres_descripcion_corta'] );
			}

			if( isset( $options['cw_texto_boton_login'] ) ) {
				update_option( 'wglop_texto_boton_login', $options['cw_texto_boton_login'] );
			}

			if( isset( $options['cw_texto_boton_add_cart_modal'] ) ) {
				update_option( 'wglop_texto_boton_add_to_cart', $options['cw_texto_boton_add_cart_modal'] );
			}

			if( isset( $options['cw_texto_boton_final_compra'] ) ) {
				update_option( 'wglop_texto_boton_finalizar_compra', $options['cw_texto_boton_final_compra'] );
			}

			if( isset( $options['cw_accent_color'] ) ) {
				set_theme_mod( 'accent_color', $options['cw_accent_color'] );
			}

			if( isset( $options['cw_accent_color_deliver_type'] ) ) {
				set_theme_mod( 'accent_color_deliver_type', $options['cw_accent_color_deliver_type'] );
			}

			if( isset( $options['cw_font_color_deliver_type'] ) ) {
				set_theme_mod( 'font_color_deliver_type', $options['cw_font_color_deliver_type'] );
			}

			if( isset( $options['cw_borrar_todo'] ) && 1 === intval( $options['cw_borrar_todo'] ) ) {
				// BORRAR CATEGORÍAS Y PRODUCTOS
				$allposts= get_posts( array('post_type'=>'product','numberposts'=>-1) );
				foreach ($allposts as $eachpost) {
					wp_delete_post( $eachpost->ID, true );
				}

				$taxonomy_name = 'product_cat';
				$terms = get_terms( array(
					'taxonomy' => $taxonomy_name,
					'hide_empty' => false
				) );
				foreach ( $terms as $term ) {
					wp_delete_term($term->term_id, $taxonomy_name);
				}
			}

		}catch (Exception $e) {
			if(getenv("REMOTE_ADDR") == '62.57.20.247') {
				echo "<textarea>";print_r($e);echo "</textarea>";exit();
			}
		}
		return rest_ensure_response( array() );

	}

	/**
	 * SKU de productos.
	 *
	 * @param array $alergenos Productos.
	 */
	private function set_alergenos( $alergenos ) {
		if ( ! is_array( $alergenos ) || 0 === count( $alergenos ) ) {
			return;
		}

		$alergenos_options = array();

		foreach ( $alergenos as $alergeno ) {
			// guardar file en uploads.
			if ( $alergeno['src'] ) {
				$upload = Wooglop_Product::upload_product_image( $alergeno );
				$alergenos_options[ sanitize_title( $alergeno['nombre'] ) ] = $upload;
			}
		}

		update_option( 'wglop_alergenos', $alergenos_options );
	}

	/**
	 * SKU de productos.
	 *
	 * @param array $productos Productos.
	 */
	private function set_top_ventas( $productos ) {
		/**
		 * Producto.
		 *
		 * @var WC_Product[] $products Product.
		 */
		$products = wc_get_products(
			array(
				'featured'   => true,
				'visibility' => 'featured',
			)
		);

		if ( isset( $products ) && 0 < count( $products ) ) {
			foreach ( $products as $product ) {
				$product->set_featured( false );
				$product->save();
			}
		}

		global $wpdb;

		$menu_orders = wp_list_pluck(
			$wpdb->get_results(
				"SELECT ID, menu_order FROM {$wpdb->posts} WHERE post_type = 'product' ORDER BY menu_order ASC, post_title ASC"
			),
			'menu_order',
			'ID'
		);
		$_pf         = new WC_Product_Factory();
		foreach ( $productos as $key => $producto ) {
			$producto = $productos[ $key ];
			$product  = wc_get_product_id_by_sku( $producto );

			if ( ! isset( $product ) ) {
				continue;
			}
			/**
			 * Producto.
			 *
			 * @var WC_Product $product Product.
			 */
			$producto = $_pf->get_product( $product );
			unset( $menu_orders[ $producto->get_id() ] );
		}

		foreach ( $productos as $key => $producto ) {
			$producto = $productos[ $key ];
			$product  = wc_get_product_id_by_sku( $producto );

			if ( ! isset( $product ) ) {
				continue;
			}
			/**
			 * Producto.
			 *
			 * @var WC_Product $product Product.
			 */
			$producto = $_pf->get_product( $product );
			$producto->set_featured( true );
			$producto->save();
			$args['id'] = $producto->get_id();
			if ( 0 < $key ) {
				$product_prev = wc_get_product_id_by_sku( $productos[ $key - 1 ] );
				if ( ! isset( $product_prev ) ) {
					continue;
				}
				$args['previd'] = $product_prev;
			}
			if ( count( $productos ) - 1 === $key ) {
				$first          = array_keys( $menu_orders );
				$args['nextid'] = current( $first );
			} else {
				$product_next = wc_get_product_id_by_sku( $productos[ $key + 1 ] );
				if ( ! isset( $product_next ) ) {
					continue;
				}
				if ( 0 < $key ) {
					$args['nextid'] = $product_next;
				} else {
					$first = array_keys( $menu_orders );
					reset( $first );
					$args['nextid'] = current( $first );
				}
			}

			$this->wooglop_product_ordering( $args );

		}
	}

	/**
	 *
	 * Ordenar productos.
	 *
	 * @param array $args Argumentos.
	 */
	private function wooglop_product_ordering( $args ) {
		// phpcs:disable
		global $wpdb;
		$sorting_id  = absint( $args['id'] );
		$previd      = absint( isset( $args['previd'] ) ? $args['previd'] : 0 );
		$nextid      = absint( isset( $args['nextid'] ) ? $args['nextid'] : 0 );

		$menu_orders = wp_list_pluck( $wpdb->get_results( "SELECT ID, menu_order FROM {$wpdb->posts} WHERE post_type = 'product' ORDER BY menu_order ASC, post_title ASC" ), 'menu_order', 'ID' );
		$index       = 0;

		foreach ( $menu_orders as $id => $menu_order ) {
			$id = absint( $id );

			if ( $sorting_id === $id ) {
				continue;
			}
			if ( $nextid === $id ) {
				$index ++;
			}
			$index ++;
			$menu_orders[ $id ] = $index;
			$wpdb->update( $wpdb->posts, array( 'menu_order' => $index ), array( 'ID' => $id ) );

			/**
			 * When a single product has gotten it's ordering updated.
			 * $id The product ID
			 * $index The new menu order
			 */
			do_action( 'woocommerce_after_single_product_ordering', $id, $index );
		}

		if ( isset( $menu_orders[ $previd ] ) ) {
			$menu_orders[ $sorting_id ] = $menu_orders[ $previd ] + 1;
		} elseif ( isset( $menu_orders[ $nextid ] ) ) {
			$menu_orders[ $sorting_id ] = $menu_orders[ $nextid ] - 1;
		} else {
			$menu_orders[ $sorting_id ] = 0;
		}

		$wpdb->update( $wpdb->posts, array( 'menu_order' => $menu_orders[ $sorting_id ] ), array( 'ID' => $sorting_id ) );

		WC_Post_Data::delete_product_query_transients();

		do_action( 'woocommerce_after_product_ordering', $sorting_id, $menu_orders );
		// phpcs:enable
	}

	/**
	 * Get admin user id.
	 *
	 * @return int
	 */
	private function wooglop_admin_user_id() {
		global $wpdb;

		$wp_user_search = $wpdb->get_results( "SELECT ID, display_name FROM $wpdb->users ORDER BY ID" );
		foreach ( $wp_user_search as $userid ) {
			$curuser    = get_userdata( $userid->ID );
			$user_level = $curuser->user_level;
			if ( 8 <= $user_level ) {
				return $userid->ID;
			}
		}
		return null;
	}

	/**
	 * Reordenar categorias.
	 *
	 * @param Array $categorias Categorias.
	 * @return mixed|WP_Error|WP_HTTP_Response|WP_REST_Response
	 */
	private function reordenar_categorias( $categorias ) {
		// Obtener categorías de woocommerce.
		$prepared_args = array(
			'exclude'    => array(),
			'include'    => array(),
			'order'      => 'asc',
			'orderby'    => 'menu_order',
			'product'    => null,
			'hide_empty' => false,
			'number'     => 99,
			'search'     => null,
			'slug'       => null,
			'offset'     => 0,
		);
		$query_result  = get_terms( 'product_cat', $prepared_args );
		$orden         = array();
		foreach ( $categorias as $categoria ) {
			foreach ( $query_result as $key => $term ) {
				$nombre = $term->name;
				if ( strtolower( $nombre ) === strtolower( $categoria ) ) {
					$orden[] = $term;
				}
			}
		}

		$resto = array();
		foreach ( $query_result as $key => $term ) {
			if ( false === array_search( strtolower( $term->name ), array_map( 'strtolower', $categorias ), true ) ) {
				$resto[] = $term;
			}
		}

		$orden = array_merge( $orden, $resto );

		/**
		 * Ordern actual.
		 *
		 * @var WP_Term[] $orden
		 */
//		foreach ( $orden as $key => $item ) {
		for ( $key = count( $orden ); $key > -1; $key-- ) {
			$item    = $orden[ $key ];
			$next_id = null;
			if ( $key + 1 < count( $orden ) ) {
				$next_id = $orden[ $key + 1 ]->term_id;
			}
			wc_set_term_order( $item->term_id, $key, 'product_cat', true );
		}

		return rest_ensure_response( $orden );
	}
}
